@@ -1,6 +1,6 @@
 
 /*
- * $Id: squid.h,v 1.85 1996/12/05 21:23:56 wessels Exp $
+ * $Id: squid.h,v 1.87 1996/12/20 23:45:44 wessels Exp $
  *
  * AUTHOR: Duane Wessels
  *
@@ -51,18 +51,6 @@
 #endif
 #endif /* _SQUID_LINUX_ */
 
-/*
- * So if FD_SETSIZE is less than SQUID_MAXFD we'd probably better
- * shrink SQUID_MAXFD so that select(2) doesn't puke even though
- * we might really be able to open more than FD_SETSIZE descriptors.
- * happy happy happy joy joy joy.
- */
-
-#if FD_SETSIZE < SQUID_MAXFD
-#undef SQUID_MAXFD
-#define SQUID_MAXFD FD_SETSIZE
-#endif
-
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -195,10 +183,6 @@
 #define MAXPATHLEN SQUID_MAXPATHLEN
 #endif
 
-#ifndef INADDR_NONE
-#define INADDR_NONE ((unsigned long) -1)
-#endif
-
 #if !defined(HAVE_GETRUSAGE) && defined(_SQUID_HPUX_)
 #define HAVE_GETRUSAGE 1
 #define getrusage(a, b)  syscall(SYS_GETRUSAGE, a, b)
@@ -267,6 +251,17 @@ typedef unsigned long u_num32;
 #include "GNUregex.h"
 #endif /* HAVE_REGEX_H */
 
+/*
+ * So if FD_SETSIZE is less than SQUID_MAXFD we'd probably better
+ * shrink SQUID_MAXFD so that select(2) doesn't puke even though
+ * we might really be able to open more than FD_SETSIZE descriptors.
+ * happy happy happy joy joy joy.
+ */
+#if FD_SETSIZE < SQUID_MAXFD
+#undef SQUID_MAXFD
+#define SQUID_MAXFD FD_SETSIZE
+#endif
+
 typedef void (*SIH) (int, void *);	/* swap in */
 typedef int (*QS) (const void *, const void *);
 
@@ -337,6 +332,7 @@ extern const char *const appname;	/* main.c */
 extern struct in_addr local_addr;	/* main.c */
 extern struct in_addr theOutICPAddr;	/* main.c */
 extern const char *const localhost;
+extern unsigned int inaddr_none;
 extern struct in_addr any_addr;	/* comm.c */
 extern struct in_addr no_addr;	/* comm.c */
 extern int opt_udp_hit_obj;	/* main.c */
