@@ -1,11 +1,11 @@
 
 /*
- * $Id: ssl.c,v 1.1.2.24 1996/08/22 16:50:07 wessels Exp $
+ * $Id: ssl.c,v 1.31 1996/12/17 07:16:57 wessels Exp $
  *
  * DEBUG: section 26    Secure Sockets Layer Proxy
  * AUTHOR: Duane Wessels
  *
- * SQUID Internet Object Cache  http://www.nlanr.net/Squid/
+ * SQUID Internet Object Cache  http://squid.nlanr.net/Squid/
  * --------------------------------------------------------
  *
  *  Squid is the result of efforts by numerous individuals from the
@@ -33,6 +33,8 @@
 
 typedef struct {
     char *url;
+    char *host;			/* either request->host or proxy host */
+    u_short port;
     request_t *request;
     char *mime_hdr;
     struct {
@@ -43,30 +45,35 @@ typedef struct {
     } client, server;
     time_t timeout;
     int *size_ptr;		/* pointer to size in an icpStateData for logging */
+    ConnectStateData connectState;
+    int proxying;
 } SslStateData;
 
-static char conn_established[] = "HTTP/1.0 200 Connection established\r\n\r\n";
+static const char *const conn_established = "HTTP/1.0 200 Connection established\r\n\r\n";
 
-static void sslLifetimeExpire _PARAMS((int fd, SslStateData * sslState));
-static void sslReadTimeout _PARAMS((int fd, SslStateData * sslState));
-static void sslReadServer _PARAMS((int fd, SslStateData * sslState));
-static void sslReadClient _PARAMS((int fd, SslStateData * sslState));
-static void sslWriteServer _PARAMS((int fd, SslStateData * sslState));
-static void sslWriteClient _PARAMS((int fd, SslStateData * sslState));
-static void sslConnected _PARAMS((int fd, SslStateData * sslState));
-static int sslConnect _PARAMS((int fd, struct hostent *, SslStateData *));
-static void sslConnInProgress _PARAMS((int fd, SslStateData * sslState));
+static void sslLifetimeExpire _PARAMS((int fd, void *));
+static void sslReadTimeout _PARAMS((int fd, void *));
+static void sslReadServer _PARAMS((int fd, void *));
+static void sslReadClient _PARAMS((int fd, void *));
+static void sslWriteServer _PARAMS((int fd, void *));
+static void sslWriteClient _PARAMS((int fd, void *));
+static void sslConnected _PARAMS((int fd, void *));
+static void sslProxyConnected _PARAMS((int fd, void *));
+static void sslConnect _PARAMS((int fd, const ipcache_addrs *, void *));
 static void sslErrorComplete _PARAMS((int, char *, int, int, void *));
 static void sslClose _PARAMS((SslStateData * sslState));
-static int sslClientClosed _PARAMS((int fd, SslStateData * sslState));
+static void sslClientClosed _PARAMS((int fd, void *));
+static void sslConnectDone _PARAMS((int fd, int status, void *data));
+static void sslStateFree _PARAMS((int fd, void *data));
+static void sslSelectNeighbor _PARAMS((int fd, const ipcache_addrs *, void *));
 
-static void sslClose(sslState)
-     SslStateData *sslState;
+static void
+sslClose(SslStateData * sslState)
 {
     if (sslState->client.fd > -1) {
 	/* remove the "unexpected" client close handler */
 	comm_remove_close_handler(sslState->client.fd,
-	    (PF) sslClientClosed,
+	    sslClientClosed,
 	    (void *) sslState);
 	comm_close(sslState->client.fd);
 	sslState->client.fd = -1;
@@ -78,74 +85,72 @@ static void sslClose(sslState)
 
 /* This is called only if the client connect closes unexpectedly,
  * ie from icpDetectClientClose() */
-static int sslClientClosed(fd, sslState)
-     int fd;
-     SslStateData *sslState;
+static void
+sslClientClosed(int fd, void *data)
 {
+    SslStateData *sslState = data;
     debug(26, 3, "sslClientClosed: FD %d\n", fd);
     /* we have been called from comm_close for the client side, so
      * just need to clean up the server side */
-    protoUndispatch(sslState->server.fd,
-	sslState->url,
+    protoUnregister(sslState->server.fd,
 	NULL,
-	sslState->request);
+	sslState->request,
+	no_addr);
     comm_close(sslState->server.fd);
-    return 0;
 }
 
-static int sslStateFree(fd, sslState)
-     int fd;
-     SslStateData *sslState;
+static void
+sslStateFree(int fd, void *data)
 {
+    SslStateData *sslState = data;
     debug(26, 3, "sslStateFree: FD %d, sslState=%p\n", fd, sslState);
     if (sslState == NULL)
-	return 1;
+	return;
     if (fd != sslState->server.fd)
 	fatal_dump("sslStateFree: FD mismatch!\n");
-    comm_set_select_handler(sslState->client.fd,
+    commSetSelect(sslState->client.fd,
 	COMM_SELECT_READ,
 	NULL,
-	NULL);
+	NULL, 0);
     safe_free(sslState->server.buf);
     safe_free(sslState->client.buf);
     xfree(sslState->url);
     requestUnlink(sslState->request);
     memset(sslState, '\0', sizeof(SslStateData));
     safe_free(sslState);
-    return 0;
 }
 
 /* This will be called when the server lifetime is expired. */
-static void sslLifetimeExpire(fd, sslState)
-     int fd;
-     SslStateData *sslState;
+static void
+sslLifetimeExpire(int fd, void *data)
 {
+    SslStateData *sslState = data;
     debug(26, 4, "sslLifeTimeExpire: FD %d: URL '%s'>\n",
 	fd, sslState->url);
     sslClose(sslState);
 }
 
 /* Read from server side and queue it for writing to the client */
-static void sslReadServer(fd, sslState)
-     int fd;
-     SslStateData *sslState;
+static void
+sslReadServer(int fd, void *data)
 {
+    SslStateData *sslState = data;
     int len;
     len = read(sslState->server.fd, sslState->server.buf, SQUID_TCP_SO_RCVBUF);
     debug(26, 5, "sslReadServer FD %d, read %d bytes\n", fd, len);
     if (len < 0) {
-	debug(26, 1, "sslReadServer: FD %d: read failure: %s\n",
+	debug(50, 1, "sslReadServer: FD %d: read failure: %s\n",
 	    sslState->server.fd, xstrerror());
 	if (errno == EAGAIN || errno == EWOULDBLOCK) {
 	    /* reinstall handlers */
 	    /* XXX This may loop forever */
-	    comm_set_select_handler(sslState->server.fd,
+	    commSetSelect(sslState->server.fd,
 		COMM_SELECT_READ,
-		(PF) sslReadServer,
-		(void *) sslState);
-	    comm_set_select_handler_plus_timeout(sslState->server.fd,
+		sslReadServer,
+		(void *) sslState, 0);
+	    commSetSelect(sslState->server.fd,
 		COMM_SELECT_TIMEOUT,
-		(PF) sslReadTimeout,
+		sslReadTimeout,
 		(void *) sslState,
 		sslState->timeout);
 	} else {
@@ -157,32 +162,32 @@ static void sslReadServer(fd, sslState)
     } else {
 	sslState->server.offset = 0;
 	sslState->server.len = len;
-	comm_set_select_handler(sslState->client.fd,
+	commSetSelect(sslState->client.fd,
 	    COMM_SELECT_WRITE,
-	    (PF) sslWriteClient,
-	    (void *) sslState);
+	    sslWriteClient,
+	    (void *) sslState, 0);
     }
 }
 
 /* Read from client side and queue it for writing to the server */
-static void sslReadClient(fd, sslState)
-     int fd;
-     SslStateData *sslState;
+static void
+sslReadClient(int fd, void *data)
 {
+    SslStateData *sslState = data;
     int len;
     len = read(sslState->client.fd, sslState->client.buf, SQUID_TCP_SO_RCVBUF);
     debug(26, 5, "sslReadClient FD %d, read %d bytes\n",
 	sslState->client.fd, len);
     if (len < 0) {
-	debug(26, 1, "sslReadClient: FD %d: read failure: %s\n",
+	debug(50, 1, "sslReadClient: FD %d: read failure: %s\n",
 	    fd, xstrerror());
 	if (errno == EAGAIN || errno == EWOULDBLOCK) {
 	    /* reinstall handlers */
 	    /* XXX This may loop forever */
-	    comm_set_select_handler(sslState->client.fd,
+	    commSetSelect(sslState->client.fd,
 		COMM_SELECT_READ,
-		(PF) sslReadClient,
-		(void *) sslState);
+		sslReadClient,
+		(void *) sslState, 0);
 	} else {
 	    sslClose(sslState);
 	}
@@ -192,54 +197,54 @@ static void sslReadClient(fd, sslState)
     } else {
 	sslState->client.offset = 0;
 	sslState->client.len = len;
-	comm_set_select_handler(sslState->server.fd,
+	commSetSelect(sslState->server.fd,
 	    COMM_SELECT_WRITE,
-	    (PF) sslWriteServer,
-	    (void *) sslState);
+	    sslWriteServer,
+	    (void *) sslState, 0);
     }
 }
 
 /* Writes data from the client buffer to the server side */
-static void sslWriteServer(fd, sslState)
-     int fd;
-     SslStateData *sslState;
+static void
+sslWriteServer(int fd, void *data)
 {
+    SslStateData *sslState = data;
     int len;
     len = write(sslState->server.fd,
 	sslState->client.buf + sslState->client.offset,
 	sslState->client.len - sslState->client.offset);
     debug(26, 5, "sslWriteServer FD %d, wrote %d bytes\n", fd, len);
     if (len < 0) {
-	debug(26, 2, "sslWriteServer: FD %d: write failure: %s.\n",
+	debug(50, 2, "sslWriteServer: FD %d: write failure: %s.\n",
 	    sslState->server.fd, xstrerror());
 	sslClose(sslState);
 	return;
     }
     if ((sslState->client.offset += len) >= sslState->client.len) {
 	/* Done writing, read more */
-	comm_set_select_handler(sslState->client.fd,
+	commSetSelect(sslState->client.fd,
 	    COMM_SELECT_READ,
-	    (PF) sslReadClient,
-	    (void *) sslState);
-	comm_set_select_handler_plus_timeout(sslState->server.fd,
+	    sslReadClient,
+	    (void *) sslState, 0);
+	commSetSelect(sslState->server.fd,
 	    COMM_SELECT_TIMEOUT,
-	    (PF) sslReadTimeout,
+	    sslReadTimeout,
 	    (void *) sslState,
 	    sslState->timeout);
     } else {
 	/* still have more to write */
-	comm_set_select_handler(sslState->server.fd,
+	commSetSelect(sslState->server.fd,
 	    COMM_SELECT_WRITE,
-	    (PF) sslWriteServer,
-	    (void *) sslState);
+	    sslWriteServer,
+	    (void *) sslState, 0);
     }
 }
 
 /* Writes data from the server buffer to the client side */
-static void sslWriteClient(fd, sslState)
-     int fd;
-     SslStateData *sslState;
+static void
+sslWriteClient(int fd, void *data)
 {
+    SslStateData *sslState = data;
     int len;
     debug(26, 5, "sslWriteClient FD %d len=%d offset=%d\n",
 	fd,
@@ -250,7 +255,7 @@ static void sslWriteClient(fd, sslState)
 	sslState->server.len - sslState->server.offset);
     debug(26, 5, "sslWriteClient FD %d, wrote %d bytes\n", fd, len);
     if (len < 0) {
-	debug(26, 2, "sslWriteClient: FD %d: write failure: %s.\n",
+	debug(50, 2, "sslWriteClient: FD %d: write failure: %s.\n",
 	    sslState->client.fd, xstrerror());
 	sslClose(sslState);
 	return;
@@ -259,109 +264,62 @@ static void sslWriteClient(fd, sslState)
 	*sslState->size_ptr += len;	/* increment total object size */
     if ((sslState->server.offset += len) >= sslState->server.len) {
 	/* Done writing, read more */
-	comm_set_select_handler(sslState->server.fd,
+	commSetSelect(sslState->server.fd,
 	    COMM_SELECT_READ,
-	    (PF) sslReadServer,
-	    (void *) sslState);
+	    sslReadServer,
+	    (void *) sslState, 0);
     } else {
 	/* still have more to write */
-	comm_set_select_handler(sslState->client.fd,
+	commSetSelect(sslState->client.fd,
 	    COMM_SELECT_WRITE,
-	    (PF) sslWriteClient,
-	    (void *) sslState);
+	    sslWriteClient,
+	    (void *) sslState, 0);
     }
 }
 
-static void sslReadTimeout(fd, sslState)
-     int fd;
-     SslStateData *sslState;
+static void
+sslReadTimeout(int fd, void *data)
 {
+    SslStateData *sslState = data;
     debug(26, 3, "sslReadTimeout: FD %d\n", fd);
     sslClose(sslState);
 }
 
-static void sslConnected(fd, sslState)
-     int fd;
-     SslStateData *sslState;
+static void
+sslConnected(int fd, void *data)
 {
+    SslStateData *sslState = data;
     debug(26, 3, "sslConnected: FD %d sslState=%p\n", fd, sslState);
     strcpy(sslState->server.buf, conn_established);
     sslState->server.len = strlen(conn_established);
     sslState->server.offset = 0;
-    comm_set_select_handler(sslState->client.fd,
+    commSetSelect(sslState->client.fd,
 	COMM_SELECT_WRITE,
-	(PF) sslWriteClient,
-	(void *) sslState);
+	sslWriteClient,
+	(void *) sslState, 0);
     comm_set_fd_lifetime(fd, 86400);	/* extend lifetime */
-    comm_set_select_handler(sslState->client.fd,
+    commSetSelect(sslState->client.fd,
 	COMM_SELECT_READ,
-	(PF) sslReadClient,
-	(void *) sslState);
+	sslReadClient,
+	(void *) sslState, 0);
 }
 
-static void sslErrorComplete(fd, buf, size, errflag, sslState)
-     int fd;
-     char *buf;
-     int size;
-     int errflag;
-     void *sslState;
+static void
+sslErrorComplete(int fd, char *buf, int size, int errflag, void *sslState)
 {
     safe_free(buf);
     sslClose(sslState);
 }
 
 
-static void sslConnInProgress(fd, sslState)
-     int fd;
-     SslStateData *sslState;
-{
-    request_t *req = sslState->request;
-    char *buf = NULL;
-    debug(26, 5, "sslConnInProgress: FD %d sslState=%p\n", fd, sslState);
-
-    if (comm_connect(fd, req->host, req->port) != COMM_OK) {
-	debug(26, 5, "sslConnInProgress: FD %d: %s\n", fd, xstrerror());
-	switch (errno) {
-#if EINPROGRESS != EALREADY
-	case EINPROGRESS:
-#endif
-	case EALREADY:
-	    /* We are not connected yet. schedule this handler again */
-	    comm_set_select_handler(fd, COMM_SELECT_WRITE,
-		(PF) sslConnInProgress,
-		(void *) sslState);
-	    return;
-	default:
-	    buf = squid_error_url(sslState->url,
-		METHOD_CONNECT,
-		ERR_CONNECT_FAIL,
-		NULL,
-		500,
-		xstrerror());
-	    comm_write(sslState->client.fd,
-		xstrdup(buf),
-		strlen(buf),
-		30,
-		sslErrorComplete,
-		sslState);
-	    return;
-	}
-    }
-    /* We are now fully connected */
-    sslConnected(fd, sslState);
-    return;
-}
-
-static int sslConnect(fd, hp, sslState)
-     int fd;
-     struct hostent *hp;
-     SslStateData *sslState;
+static void
+sslConnect(int fd, const ipcache_addrs * ia, void *data)
 {
+    SslStateData *sslState = data;
     request_t *request = sslState->request;
-    int status;
     char *buf = NULL;
-    if (hp == NULL) {
-	debug(26, 4, "sslConnect: Unknown host: %s\n", request->host);
+    if (ia == NULL) {
+	debug(26, 4, "sslConnect: Unknown host: %s\n", sslState->host);
 	buf = squid_error_url(sslState->url,
 	    request->method,
 	    ERR_DNS_FAIL,
@@ -373,61 +331,67 @@ static int sslConnect(fd, hp, sslState)
 	    strlen(buf),
 	    30,
 	    sslErrorComplete,
-	    (void *) sslState);
-	return COMM_ERROR;
+	    (void *) sslState,
+	    xfree);
+	return;
     }
     debug(26, 5, "sslConnect: client=%d server=%d\n",
 	sslState->client.fd,
 	sslState->server.fd);
     /* Install lifetime handler */
-    comm_set_select_handler(sslState->server.fd,
+    commSetSelect(sslState->server.fd,
 	COMM_SELECT_LIFETIME,
-	(PF) sslLifetimeExpire,
-	(void *) sslState);
+	sslLifetimeExpire,
+	(void *) sslState, 0);
     /* NOTE this changes the lifetime handler for the client side.
      * It used to be asciiConnLifetimeHandle, but it does funny things
      * like looking for read handlers and assuming it was still reading
      * the HTTP request.  sigh... */
-    comm_set_select_handler(sslState->client.fd,
+    commSetSelect(sslState->client.fd,
 	COMM_SELECT_LIFETIME,
-	(PF) sslLifetimeExpire,
-	(void *) sslState);
-    /* Open connection. */
-    if ((status = comm_connect(fd, request->host, request->port))) {
-	if (status != EINPROGRESS) {
-	    buf = squid_error_url(sslState->url,
-		request->method,
-		ERR_CONNECT_FAIL,
-		fd_table[fd].ipaddr,
-		500,
-		xstrerror());
-	    comm_write(sslState->client.fd,
-		xstrdup(buf),
-		strlen(buf),
-		30,
-		sslErrorComplete,
-		(void *) sslState);
-	    return COMM_ERROR;
-	} else {
-	    debug(26, 5, "sslConnect: conn %d EINPROGRESS\n", fd);
-	    /* The connection is in progress, install ssl handler */
-	    comm_set_select_handler(sslState->server.fd,
-		COMM_SELECT_WRITE,
-		(PF) sslConnInProgress,
-		(void *) sslState);
-	    return COMM_OK;
-	}
-    }
-    sslConnected(sslState->server.fd, sslState);
-    return COMM_OK;
+	sslLifetimeExpire,
+	(void *) sslState, 0);
+    sslState->connectState.fd = fd;
+    sslState->connectState.host = sslState->host;
+    sslState->connectState.port = sslState->port;
+    sslState->connectState.handler = sslConnectDone;
+    sslState->connectState.data = sslState;
+    comm_nbconnect(fd, &sslState->connectState);
 }
 
-int sslStart(fd, url, request, mime_hdr, size_ptr)
-     int fd;
-     char *url;
-     request_t *request;
-     char *mime_hdr;
-     int *size_ptr;
+static void
+sslConnectDone(int fd, int status, void *data)
+{
+    SslStateData *sslState = data;
+    char *buf = NULL;
+    if (status == COMM_ERROR) {
+	buf = squid_error_url(sslState->url,
+	    sslState->request->method,
+	    ERR_CONNECT_FAIL,
+	    fd_table[fd].ipaddr,
+	    500,
+	    xstrerror());
+	comm_write(sslState->client.fd,
+	    xstrdup(buf),
+	    strlen(buf),
+	    30,
+	    sslErrorComplete,
+	    (void *) sslState,
+	    xfree);
+	return;
+    }
+    if (opt_no_ipcache)
+	ipcacheInvalidate(sslState->host);
+    if (sslState->proxying)
+	sslProxyConnected(sslState->server.fd, sslState);
+    else
+	sslConnected(sslState->server.fd, sslState);
+    if (vizSock > -1)
+	vizHackSendPkt(&sslState->connectState.S, 2);
+}
+
+int
+sslStart(int fd, const char *url, request_t * request, char *mime_hdr, int *size_ptr)
 {
     /* Create state structure. */
     SslStateData *sslState = NULL;
@@ -438,7 +402,12 @@ int sslStart(fd, url, request, mime_hdr, size_ptr)
 	RequestMethodStr[request->method], url);
 
     /* Create socket. */
-    sock = comm_open(COMM_NONBLOCKING, getTcpOutgoingAddr(), 0, url);
+    sock = comm_open(SOCK_STREAM,
+	0,
+	Config.Addrs.tcp_outgoing,
+	0,
+	COMM_NONBLOCKING,
+	url);
     if (sock == COMM_ERROR) {
 	debug(26, 4, "sslStart: Failed because we're out of sockets.\n");
 	buf = squid_error_url(url,
@@ -452,28 +421,99 @@ int sslStart(fd, url, request, mime_hdr, size_ptr)
 	    strlen(buf),
 	    30,
 	    sslErrorComplete,
-	    (void *) sslState);
+	    (void *) sslState,
+	    xfree);
 	return COMM_ERROR;
     }
     sslState = xcalloc(1, sizeof(SslStateData));
     sslState->url = xstrdup(url);
     sslState->request = requestLink(request);
     sslState->mime_hdr = mime_hdr;
-    sslState->timeout = getReadTimeout();
+    sslState->timeout = Config.readTimeout;
     sslState->size_ptr = size_ptr;
     sslState->client.fd = fd;
     sslState->server.fd = sock;
     sslState->server.buf = xmalloc(SQUID_TCP_SO_RCVBUF);
     sslState->client.buf = xmalloc(SQUID_TCP_SO_RCVBUF);
     comm_add_close_handler(sslState->server.fd,
-	(PF) sslStateFree,
+	sslStateFree,
 	(void *) sslState);
     comm_add_close_handler(sslState->client.fd,
-	(PF) sslClientClosed,
+	sslClientClosed,
 	(void *) sslState);
-    ipcache_nbgethostbyname(request->host,
-	sslState->server.fd,
-	(IPH) sslConnect,
-	sslState);
+
+    if (Config.sslProxy) {
+	ipcache_nbgethostbyname(request->host,
+	    sslState->server.fd,
+	    sslSelectNeighbor,
+	    sslState);
+    } else {
+	sslState->host = request->host;
+	sslState->port = request->port;
+	ipcache_nbgethostbyname(request->host,
+	    sslState->server.fd,
+	    sslConnect,
+	    sslState);
+    }
     return COMM_OK;
 }
+
+static void
+sslProxyConnected(int fd, void *data)
+{
+    SslStateData *sslState = data;
+    debug(26, 3, "sslProxyConnected: FD %d sslState=%p\n", fd, sslState);
+    sprintf(sslState->client.buf, "CONNECT %s HTTP/1.0\r\n\r\n", sslState->url);
+    debug(26, 3, "sslProxyConnected: Sending 'CONNECT %s HTTP/1.0'\n", sslState->url);
+    sslState->client.len = strlen(sslState->client.buf);
+    sslState->client.offset = 0;
+    commSetSelect(sslState->server.fd,
+	COMM_SELECT_WRITE,
+	sslWriteServer,
+	(void *) sslState, 0);
+    comm_set_fd_lifetime(fd, 86400);	/* extend lifetime */
+    commSetSelect(sslState->server.fd,
+	COMM_SELECT_READ,
+	sslReadServer,
+	(void *) sslState, 0);
+}
+
+static void
+sslSelectNeighbor(int fd, const ipcache_addrs * ia, void *data)
+{
+    SslStateData *sslState = data;
+    request_t *request = sslState->request;
+    edge *e = NULL;
+    edge *g = NULL;
+    int fw_ip_match = IP_ALLOW;
+    if (ia && Config.firewall_ip_list)
+	fw_ip_match = ip_access_check(ia->in_addrs[ia->cur], Config.firewall_ip_list);
+    if ((e = Config.sslProxy)) {
+	hierarchyNote(request, HIER_SSL_PARENT, 0, e->host);
+    } else if (matchInsideFirewall(request->host)) {
+	hierarchyNote(request, HIER_DIRECT, 0, request->host);
+    } else if (fw_ip_match == IP_DENY) {
+	hierarchyNote(request, HIER_DIRECT, 0, request->host);
+    } else if ((e = getDefaultParent(request))) {
+	hierarchyNote(request, HIER_DEFAULT_PARENT, 0, e->host);
+    } else if ((e = getSingleParent(request))) {
+	hierarchyNote(request, HIER_SINGLE_PARENT, 0, e->host);
+    } else if ((e = getFirstUpParent(request))) {
+	hierarchyNote(request, HIER_FIRSTUP_PARENT, 0, e->host);
+    }
+    sslState->proxying = e ? 1 : 0;
+    sslState->host = e ? e->host : request->host;
+    if (e == NULL) {
+	sslState->port = request->port;
+    } else if (e->http_port != 0) {
+	sslState->port = e->http_port;
+    } else if ((g = neighborFindByName(e->host))) {
+	sslState->port = g->http_port;
+    } else {
+	sslState->port = CACHE_HTTP_PORT;
+    }
+    ipcache_nbgethostbyname(sslState->host,
+	sslState->server.fd,
+	sslConnect,
+	sslState);
+}
